{% extends "admin/change_form.html" %}

{% load i18n admin_urls %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
&rsaquo; {% if has_change_permission %}<a href="{% url opts|admin_urlname:'changelist' %}">{% trans 'Root folder' %}</a>{% else %}{% trans 'Root folder' %}{% endif %}
{% if original %}
  {% for f in original.ancestors_including_self %}
  &rsaquo; <a href="{% url opts|admin_urlname:'changelist' %}?folder__id__exact={{ f.id }}">{{ f }}</a>
  {% endfor %}
{% elif add %}
  &rsaquo; {% if add %}{% blocktrans with name=_('folder') %}Add {{ name }}{% endblocktrans %}{% endif %}
{% endif %}

</div>
{% endblock %}

{% block submit_buttons_bottom %}
<div class="submit-row">
<input type="submit" value="{% trans 'Save' %}" class="default" name="_save" />
</div>
{% endblock %}
