# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-20 06:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("cabinet", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="file",
            name="caption",
            field=models.CharField(blank=True, max_length=1000, verbose_name="caption"),
        ),
        migrations.AddField(
            model_name="file",
            name="copyright",
            field=models.CharField(
                blank=True, max_length=1000, verbose_name="copyright"
            ),
        ),
        migrations.AddField(
            model_name="file",
            name="image_alt_text",
            field=models.CharField(
                blank=True, max_length=1000, verbose_name="alternative text"
            ),
        ),
    ]
