# Generated by Django 2.2 on 2019-04-15 03:43

from django.db import migrations, models
import imagefield.fields


class Migration(migrations.Migration):

    dependencies = [("cabinet", "0004_auto_20181212_0252")]

    operations = [
        migrations.AlterField(
            model_name="file",
            name="download_file",
            field=models.FileField(
                blank=True,
                max_length=1000,
                upload_to="cabinet/%Y/%m",
                verbose_name="download",
            ),
        ),
        migrations.AlterField(
            model_name="file",
            name="image_file",
            field=imagefield.fields.ImageField(
                blank=True,
                height_field="image_height",
                max_length=1000,
                upload_to="cabinet/%Y/%m",
                verbose_name="image",
                width_field="image_width",
            ),
        ),
    ]
